==> primordial_harvest-fabric-1.21.1-21.1.2-sources.jar: net/darkhax/primordialharvest/common/impl/blocks/SpireshuckBlock.java <==
package net.darkhax.primordialharvest.common.impl.blocks;

import net.darkhax.primordialharvest.common.impl.Content;
import net.darkhax.primordialharvest.common.impl.Helper;
import net.darkhax.primordialharvest.common.impl.PrimordialHarvest;
import net.darkhax.primordialharvest.common.mixin.AccessorCropBlock;
import net.minecraft.class_1269;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2256;
import net.minecraft.class_2320;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2498;
import net.minecraft.class_259;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2756;
import net.minecraft.class_2758;
import net.minecraft.class_3218;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3545;
import net.minecraft.class_3619;
import net.minecraft.class_3620;
import net.minecraft.class_3726;
import net.minecraft.class_3727;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_5712;
import net.minecraft.class_5819;
import net.minecraft.class_7924;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SpireshuckBlock extends class_2320 implements class_2256 {

    public static final class_5321<class_52> HARVEST_TABLE = class_5321.method_29179(class_7924.field_50079, PrimordialHarvest.id("gameplay/harvest_spireshuck"));
    public static final class_2758 AGE = class_2741.field_37654;
    private static final class_265 FULL_UPPER_SHAPE;
    private static final class_265 FULL_LOWER_SHAPE;
    private static final class_265 COLLISION_SHAPE_BULB;
    private static final class_265[] UPPER_SHAPE_BY_AGE;
    private static final class_265[] LOWER_SHAPE_BY_AGE;

    static {
        FULL_UPPER_SHAPE = class_2248.method_9541(3.0, 0.0, 3.0, 13.0, 15.0, 13.0);
        FULL_LOWER_SHAPE = class_2248.method_9541(3.0, -1.0, 3.0, 13.0, 16.0, 13.0);
        COLLISION_SHAPE_BULB = class_2248.method_9541(5.0, -1.0, 5.0, 11.0, 3.0, 11.0);
        UPPER_SHAPE_BY_AGE = new class_265[]{class_2248.method_9541(3.0, 0.0, 3.0, 13.0, 11.0, 13.0), FULL_UPPER_SHAPE};
        LOWER_SHAPE_BY_AGE = new class_265[]{COLLISION_SHAPE_BULB, class_2248.method_9541(3.0, -1.0, 3.0, 13.0, 14.0, 13.0), FULL_LOWER_SHAPE, FULL_LOWER_SHAPE, FULL_LOWER_SHAPE};
    }

    public SpireshuckBlock() {
        super(class_2251.method_9637().method_31710(class_3620.field_16004).method_51370().method_9634().method_9640().method_9618().method_9626(class_2498.field_18852).method_50012(class_3619.field_15971));
    }

    @Override
    protected class_1269 method_55766(@NotNull class_2680 state, @NotNull class_1937 level, @NotNull class_2338 pos, @NotNull class_1657 player, @NotNull class_3965 hitResult) {
        if (!isLower(state) && isMaxAge(state)) {
            Helper.dropLoot(level, pos, HARVEST_TABLE, player, class_1799.field_8037);
            level.method_8396(null, pos, class_3417.field_17617, class_3419.field_15245, 1f, 0.8f + level.field_9229.method_43057() * 0.4f);
            final class_2680 newBlock = state.method_11657(AGE, 3);
            level.method_8652(pos, newBlock, 2);
            final class_2680 below = level.method_8320(pos.method_10074());
            if (isLower(below)) {
                level.method_8652(pos.method_10074(), below.method_11657(AGE, 3), 2);
            }
            level.method_43276(class_5712.field_28733, pos, class_5712.class_7397.method_43286(player, newBlock));
            return class_1269.method_29236(level.field_9236);
        }
        return super.method_55766(state, level, pos, player, hitResult);
    }

    @Override
    public class_2680 method_9605(@NotNull class_1750 context) {
        return this.method_9564();
    }

    @NotNull
    @Override
    public class_265 method_9530(class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos, @NotNull class_3726 context) {
        if (state.method_11654(field_10929) == class_2756.field_12609) {
            return UPPER_SHAPE_BY_AGE[Math.min(Math.abs(4 - (state.method_11654(AGE) + 1)), UPPER_SHAPE_BY_AGE.length - 1)];
        }
        return LOWER_SHAPE_BY_AGE[state.method_11654(AGE)];
    }

    @NotNull
    @Override
    public class_265 method_9549(@NotNull class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos, @NotNull class_3726 context) {
        return context instanceof class_3727 entContext && entContext.method_32480() instanceof class_1657 ? class_259.method_1073() : this.method_9530(state, level, pos, context);
    }

    @NotNull
    @Override
    public class_2680 method_9559(class_2680 state, @NotNull class_2350 facing, @NotNull class_2680 facingState, @NotNull class_1936 level, @NotNull class_2338 currentPos, @NotNull class_2338 facingPos) {
        return isDouble(state.method_11654(AGE)) ? super.method_9559(state, facing, facingState, level, currentPos, facingPos) : state.method_26184(level, currentPos) ? state : class_2246.field_10124.method_9564();
    }

    @Override
    public boolean method_9558(@NotNull class_2680 state, @NotNull class_4538 level, @NotNull class_2338 pos) {
        return isLower(state) && !sufficientLight(level, pos) ? false : super.method_9558(state, level, pos);
    }

    @Override
    protected boolean method_9695(class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos) {
        return state.method_27852(class_2246.field_10362);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(AGE);
        super.method_9515(builder);
    }

    @Override
    public boolean method_9616(@NotNull class_2680 state, @NotNull class_1750 useContext) {
        return false;
    }

    @Override
    public void method_9567(@NotNull class_1937 level, @NotNull class_2338 pos, @NotNull class_2680 state, @NotNull class_1309 placer, @NotNull class_1799 stack) {
    }

    @Override
    public boolean method_9542(class_2680 state) {
        return state.method_11654(field_10929) == class_2756.field_12607 && !isMaxAge(state);
    }

    @Override
    public void method_9514(@NotNull class_2680 state, @NotNull class_3218 level, @NotNull class_2338 pos, class_5819 random) {
        if (random.method_43048((int) (25f / AccessorCropBlock.getGrowthSpeed(this, level, pos)) + 1) == 0) {
            this.grow(level, state, pos, 1);
        }
    }

    private void grow(class_3218 level, class_2680 state, class_2338 pos, int ageIncrement) {
        final int growthAmount = Math.min(state.method_11654(AGE) + ageIncrement, 4);
        if (this.canGrow(level, pos, state, growthAmount)) {
            final class_2680 newState = state.method_11657(AGE, growthAmount);
            level.method_8652(pos, newState, 2);
            if (isDouble(growthAmount)) {
                level.method_8652(pos.method_10084(), newState.method_11657(field_10929, class_2756.field_12609), 3);
            }
        }
    }

    private boolean canGrowInto(class_4538 level, class_2338 pos) {
        final class_2680 state = level.method_8320(pos);
        return state.method_26215() || state.method_27852(this);
    }

    private static boolean sufficientLight(class_4538 level, class_2338 pos) {
        return AccessorCropBlock.hasSufficientLight(level, pos);
    }

    private boolean isLower(class_2680 state) {
        return state.method_27852(this) && state.method_11654(field_10929) == class_2756.field_12607;
    }

    private static boolean isDouble(int age) {
        return age >= 3;
    }

    private boolean canGrow(class_4538 reader, class_2338 pos, class_2680 state, int age) {
        return !isMaxAge(state) && sufficientLight(reader, pos) && (!isDouble(age) || canGrowInto(reader, pos.method_10084()));
    }

    public static boolean isMaxAge(class_2680 state) {
        return state.method_11654(AGE) >= 4;
    }

    public static boolean repelsMobs(class_2680 state) {
        return state.method_27852(Content.SPIRESHUCK_BLOCK.get()) && isMaxAge(state);
    }

    @Nullable
    private class_3545<class_2338, class_2680> getLowerHalf(class_4538 level, class_2338 pos, class_2680 state) {
        if (!isLower(state)) {
            final class_2338 below = pos.method_10074();
            final class_2680 belowState = level.method_8320(below);
            return isLower(belowState) ? new class_3545<>(below, belowState) : null;
        }
        return new class_3545<>(pos, state);
    }

    @Override
    public boolean method_9651(@NotNull class_4538 level, @NotNull class_2338 pos, @NotNull class_2680 state) {
        final class_3545<class_2338, class_2680> lowerHalf = this.getLowerHalf(level, pos, state);
        return lowerHalf != null && this.canGrow(level, lowerHalf.method_15442(), lowerHalf.method_15441(), lowerHalf.method_15441().method_11654(AGE) + 1);
    }

    @Override
    public boolean method_9650(@NotNull class_1937 level, @NotNull class_5819 random, @NotNull class_2338 pos, @NotNull class_2680 state) {
        return true;
    }

    @Override
    public void method_9652(@NotNull class_3218 level, @NotNull class_5819 random, @NotNull class_2338 pos, @NotNull class_2680 state) {
        final class_3545<class_2338, class_2680> lower = this.getLowerHalf(level, pos, state);
        if (lower != null) {
            this.grow(level, lower.method_15441(), lower.method_15442(), 1);
        }
    }
}

==> primordial_harvest-fabric-1.21.1-21.1.8-sources.jar: net/darkhax/primordialharvest/common/impl/blocks/SpireshuckBlock.java <==
package net.darkhax.primordialharvest.common.impl.blocks;

import net.darkhax.primordialharvest.common.impl.Content;
import net.darkhax.primordialharvest.common.impl.Helper;
import net.darkhax.primordialharvest.common.impl.PrimordialHarvest;
import net.darkhax.primordialharvest.common.mixin.AccessorCropBlock;
import net.minecraft.class_1269;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2256;
import net.minecraft.class_2320;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2498;
import net.minecraft.class_259;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2756;
import net.minecraft.class_2758;
import net.minecraft.class_3218;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3545;
import net.minecraft.class_3619;
import net.minecraft.class_3620;
import net.minecraft.class_3726;
import net.minecraft.class_3727;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_5712;
import net.minecraft.class_5819;
import net.minecraft.class_7924;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SpireshuckBlock extends class_2320 implements class_2256 {

    public static final class_5321<class_52> HARVEST_TABLE = class_5321.method_29179(class_7924.field_50079, PrimordialHarvest.id("gameplay/harvest_spireshuck"));
    public static final class_2758 AGE = class_2741.field_37654;
    private static final class_265 FULL_UPPER_SHAPE;
    private static final class_265 FULL_LOWER_SHAPE;
    private static final class_265 COLLISION_SHAPE_BULB;
    private static final class_265[] UPPER_SHAPE_BY_AGE;
    private static final class_265[] LOWER_SHAPE_BY_AGE;

    static {
        FULL_UPPER_SHAPE = class_2248.method_9541(3.0, 0.0, 3.0, 13.0, 15.0, 13.0);
        FULL_LOWER_SHAPE = class_2248.method_9541(3.0, -1.0, 3.0, 13.0, 16.0, 13.0);
        COLLISION_SHAPE_BULB = class_2248.method_9541(5.0, -1.0, 5.0, 11.0, 3.0, 11.0);
        UPPER_SHAPE_BY_AGE = new class_265[]{class_2248.method_9541(3.0, 0.0, 3.0, 13.0, 11.0, 13.0), FULL_UPPER_SHAPE};
        LOWER_SHAPE_BY_AGE = new class_265[]{COLLISION_SHAPE_BULB, class_2248.method_9541(3.0, -1.0, 3.0, 13.0, 14.0, 13.0), FULL_LOWER_SHAPE, FULL_LOWER_SHAPE, FULL_LOWER_SHAPE};
    }

    public SpireshuckBlock() {
        super(class_2251.method_9637().method_31710(class_3620.field_16004).method_51370().method_9634().method_9640().method_9618().method_9626(class_2498.field_18852).method_50012(class_3619.field_15971));
    }

    @Override
    protected class_1269 method_55766(@NotNull class_2680 state, @NotNull class_1937 level, @NotNull class_2338 pos, @NotNull class_1657 player, @NotNull class_3965 hitResult) {
        if (!isLower(state) && isMaxAge(state)) {
            Helper.dropLoot(level, pos, HARVEST_TABLE, player, class_1799.field_8037);
            level.method_8396(null, pos, class_3417.field_17617, class_3419.field_15245, 1f, 0.8f + level.field_9229.method_43057() * 0.4f);
            final class_2680 newBlock = state.method_11657(AGE, 3);
            level.method_8652(pos, newBlock, 2);
            final class_2680 below = level.method_8320(pos.method_10074());
            if (isLower(below)) {
                level.method_8652(pos.method_10074(), below.method_11657(AGE, 3), 2);
            }
            level.method_43276(class_5712.field_28733, pos, class_5712.class_7397.method_43286(player, newBlock));
            return class_1269.method_29236(level.field_9236);
        }
        return super.method_55766(state, level, pos, player, hitResult);
    }

    @Override
    public class_2680 method_9605(@NotNull class_1750 context) {
        return this.method_9564();
    }

    @NotNull
    @Override
    public class_265 method_9530(class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos, @NotNull class_3726 context) {
        if (state.method_11654(field_10929) == class_2756.field_12609) {
            return UPPER_SHAPE_BY_AGE[Math.min(Math.abs(4 - (state.method_11654(AGE) + 1)), UPPER_SHAPE_BY_AGE.length - 1)];
        }
        return LOWER_SHAPE_BY_AGE[state.method_11654(AGE)];
    }

    @NotNull
    @Override
    public class_265 method_9549(@NotNull class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos, @NotNull class_3726 context) {
        return context instanceof class_3727 entContext && entContext.method_32480() instanceof class_1657 ? class_259.method_1073() : this.method_9530(state, level, pos, context);
    }

    @NotNull
    @Override
    public class_2680 method_9559(class_2680 state, @NotNull class_2350 facing, @NotNull class_2680 facingState, @NotNull class_1936 level, @NotNull class_2338 currentPos, @NotNull class_2338 facingPos) {
        return isDouble(state.method_11654(AGE)) ? super.method_9559(state, facing, facingState, level, currentPos, facingPos) : state.method_26184(level, currentPos) ? state : class_2246.field_10124.method_9564();
    }

    @Override
    public boolean method_9558(@NotNull class_2680 state, @NotNull class_4538 level, @NotNull class_2338 pos) {
        return isLower(state) && !sufficientLight(level, pos) ? false : super.method_9558(state, level, pos);
    }

    @Override
    protected boolean method_9695(class_2680 state, @NotNull class_1922 level, @NotNull class_2338 pos) {
        return state.method_27852(class_2246.field_10362);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(AGE);
        super.method_9515(builder);
    }

    @Override
    public boolean method_9616(@NotNull class_2680 state, @NotNull class_1750 useContext) {
        return false;
    }

    @Override
    public void method_9567(@NotNull class_1937 level, @NotNull class_2338 pos, @NotNull class_2680 state, @NotNull class_1309 placer, @NotNull class_1799 stack) {
    }

    @Override
    public boolean method_9542(class_2680 state) {
        return state.method_11654(field_10929) == class_2756.field_12607 && !isMaxAge(state);
    }

    @Override
    public void method_9514(@NotNull class_2680 state, @NotNull class_3218 level, @NotNull class_2338 pos, class_5819 random) {
        if (random.method_43048((int) (25f / Helper.GET_GROWTH_SPEED.apply(state, level, pos)) + 1) == 0) {
            this.grow(level, state, pos, 1);
        }
    }

    private void grow(class_3218 level, class_2680 state, class_2338 pos, int ageIncrement) {
        final int growthAmount = Math.min(state.method_11654(AGE) + ageIncrement, 4);
        if (this.canGrow(level, pos, state, growthAmount)) {
            final class_2680 newState = state.method_11657(AGE, growthAmount);
            level.method_8652(pos, newState, 2);
            if (isDouble(growthAmount)) {
                level.method_8652(pos.method_10084(), newState.method_11657(field_10929, class_2756.field_12609), 3);
            }
        }
    }

    private boolean canGrowInto(class_4538 level, class_2338 pos) {
        final class_2680 state = level.method_8320(pos);
        return state.method_26215() || state.method_27852(this);
    }

    private static boolean sufficientLight(class_4538 level, class_2338 pos) {
        return AccessorCropBlock.hasSufficientLight(level, pos);
    }

    private boolean isLower(class_2680 state) {
        return state.method_27852(this) && state.method_11654(field_10929) == class_2756.field_12607;
    }

    private static boolean isDouble(int age) {
        return age >= 3;
    }

    private boolean canGrow(class_4538 reader, class_2338 pos, class_2680 state, int age) {
        return !isMaxAge(state) && sufficientLight(reader, pos) && (!isDouble(age) || canGrowInto(reader, pos.method_10084()));
    }

    public static boolean isMaxAge(class_2680 state) {
        return state.method_11654(AGE) >= 4;
    }

    public static boolean repelsMobs(class_2680 state) {
        return state.method_27852(Content.SPIRESHUCK_BLOCK.get()) && isMaxAge(state);
    }

    @Nullable
    private class_3545<class_2338, class_2680> getLowerHalf(class_4538 level, class_2338 pos, class_2680 state) {
        if (!isLower(state)) {
            final class_2338 below = pos.method_10074();
            final class_2680 belowState = level.method_8320(below);
            return isLower(belowState) ? new class_3545<>(below, belowState) : null;
        }
        return new class_3545<>(pos, state);
    }

    @Override
    public boolean method_9651(@NotNull class_4538 level, @NotNull class_2338 pos, @NotNull class_2680 state) {
        final class_3545<class_2338, class_2680> lowerHalf = this.getLowerHalf(level, pos, state);
        return lowerHalf != null && this.canGrow(level, lowerHalf.method_15442(), lowerHalf.method_15441(), lowerHalf.method_15441().method_11654(AGE) + 1);
    }

    @Override
    public boolean method_9650(@NotNull class_1937 level, @NotNull class_5819 random, @NotNull class_2338 pos, @NotNull class_2680 state) {
        return true;
    }

    @Override
    public void method_9652(@NotNull class_3218 level, @NotNull class_5819 random, @NotNull class_2338 pos, @NotNull class_2680 state) {
        final class_3545<class_2338, class_2680> lower = this.getLowerHalf(level, pos, state);
        if (lower != null) {
            this.grow(level, lower.method_15441(), lower.method_15442(), 1);
        }
    }
}
